//--------------------------------------------------------------------------------------
// Copyright 2011 Intel Corporation
// All Rights Reserved
//
// Permission is granted to use, copy, distribute and prepare derivative works of this
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  THIS SOFTWARE IS PROVIDED "AS IS."
// INTEL SPECIFICALLY DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, AND ALL LIABILITY,
// INCLUDING CONSEQUENTIAL AND OTHER INDIRECT DAMAGES, FOR THE USE OF THIS SOFTWARE,
// INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PROPRIETARY RIGHTS, AND INCLUDING THE
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
//--------------------------------------------------------------------------------------
#include "CPUTOSServices.h"
#include "CPUTWindowAndroid.h"
#include "CPUT_OGL.h"

#include <iostream>

#include <sys/stat.h>
#include <errno.h>
#include <string.h>
#include <libgen.h>
#include <unistd.h>

// Retrieves the current working directory
//-----------------------------------------------------------------------------
CPUTResult CPUTFileSystem::GetWorkingDirectory(cString *pPath)
{
	assert(false);
	/*
    wchar_t pPathAsTchar[CPUT_MAX_PATH];
    DWORD result = GetCurrentDirectory(CPUT_MAX_PATH, pPathAsTchar);
    ASSERT( result, _L("GetCurrentDirectory returned 0.") );
    *pPath = pPathAsTchar;
	* */
    return CPUT_SUCCESS;
}

// Sets the current working directory
//-----------------------------------------------------------------------------
CPUTResult CPUTFileSystem::SetWorkingDirectory(const cString &path)
{
	assert(false);
	/*
    bool result = SetCurrentDirectory(path.c_str());
    ASSERT( 0 != result, _L("Error setting current directory.") );
    return CPUT_SUCCESS;
	 * */
}

// Gets the location of the executable's directory
//-----------------------------------------------------------------------------
CPUTResult CPUTFileSystem::GetExecutableDirectory(cString *pExecutableDir)
{
    CPUTResult result = CPUT_SUCCESS;
    
#ifdef CPUT_OS_ANDROID
    // In Android, the assets come from a /assets/ folder in the apk
    // So this doesn't really make sense.
    // We'll just stub it with a local dir
    pExecutableDir->assign("");
#else
    // get full path and executable
    char pathBuffer[CPUT_MAX_PATH];
    ssize_t count = readlink("/proc/self/exe", pathBuffer, CPUT_MAX_PATH);
    if (count == -1) {
        DEBUG_ERROR(strerror(errno));
        result = CPUT_ERROR;
    } else if (count >= CPUT_MAX_PATH) {
        DEBUG_ERROR("path is too long");
        result = CPUT_ERROR;
    }
    pathBuffer[count] = '\0';
    
    // extract path to executable
    char *dirPath;
    dirPath = dirname(pathBuffer);
    if (dirPath == NULL) {
        DEBUG_ERROR(strerror(errno));
        result = CPUT_ERROR;
    }
    
    pExecutableDir->assign(dirPath);
    pExecutableDir->append("/");
#endif    
    return result;
}

// Split up the supplied path+fileName into its constituent parts
// 
//  this doesn't really make sense in linux as there are no drive letters or necessarily any extensions
//-----------------------------------------------------------------------------
CPUTResult CPUTFileSystem::SplitPathAndFilename(const cString &sourceFilename, cString *pDrive, cString *pDir, cString *pFileName, cString *pExtension)
{
	assert(false);
	/*
    wchar_t pSplitDrive[CPUT_MAX_PATH];
    wchar_t pSplitDirs[CPUT_MAX_PATH];
    wchar_t pSplitFile[CPUT_MAX_PATH];
    wchar_t pSplitExt[CPUT_MAX_PATH];
#if defined (UNICODE) || defined(_UNICODE)
    #define SPLITPATH _wsplitpath_s
#else
    #define SPLITPATH _splitpath_s
#endif
    int result = SPLITPATH(sourceFilename.c_str(), pSplitDrive, CPUT_MAX_PATH, pSplitDirs, CPUT_MAX_PATH, pSplitFile, CPUT_MAX_PATH, pSplitExt, CPUT_MAX_PATH);
    ASSERT( 0 == result, _L("Error splitting path") );

    // return the items the user wants
    *pDrive     = pSplitDrive;
    *pDir       = pSplitDirs;
    *pFileName  = pSplitFile;
    *pExtension = pSplitExt;
*/
    return CPUT_SUCCESS;
}

// Takes a relative/full path+fileName and returns the absolute path with drive
// letter, absolute path, fileName and extension of this file.
// Truncates total path/file length to CPUT_MAX_PATH
//-----------------------------------------------------------------------------
CPUTResult CPUTFileSystem::ResolveAbsolutePathAndFilename(const cString &fileName, cString *pResolvedPathAndFilename)
{
    CPUTResult  result   = CPUT_SUCCESS;
#ifdef CPUT_OS_ANDROID
    pResolvedPathAndFilename->assign(fileName);
#else
    char       *fullPath = NULL;

    fullPath = realpath(fileName.c_str(), NULL);
    if (fullPath == NULL) {
        DEBUG_ERROR(fileName);
        DEBUG_ERROR(strerror(errno));
        result = CPUT_ERROR;
        return result;
    }
    
    pResolvedPathAndFilename->assign(fullPath);
    
    free(fullPath); // realpath() mallocs the memory required to hold the path name, so we need to free it
#endif
    return result;
}

// Verifies that file exists at specified path
// // #### this should probably just return a bool
//-----------------------------------------------------------------------------
CPUTResult CPUTFileSystem::DoesFileExist(const cString &pathAndFilename)
{
#ifdef CPUT_OS_ANDROID
    DEBUG_PRINT("DoesFileExist stubbed");
#else
    struct stat fileAttributes;
    
    int result = stat(pathAndFilename.c_str(), &fileAttributes);
    if (result == -1) {
        DEBUG_ERROR(strerror(errno));
        result = CPUT_ERROR; 
    }
	
    if (S_ISREG(fileAttributes.st_mode)) {
        return CPUT_SUCCESS;
    }
#endif    
    return CPUT_ERROR;
}

// Verifies that directory exists.
// Returns success if the directory exists and is readable (failure may mean
// it's busy or permissions denied on win32)
//-----------------------------------------------------------------------------
CPUTResult CPUTFileSystem::DoesDirectoryExist(const cString &path)
{
#ifdef CPUT_OS_ANDROID
    // On Android, all files are in the APK and are compressed.
    // We do not have access to the standard file system, so
    // all files need streaming from memory through the android asset manager
    AAssetManager* assetManager = CPUTWindowAndroid::GetAppState()->activity->assetManager;
    
    AAssetDir* AssetDir = AAssetManager_openDir(assetManager, path.c_str());
    if (AssetDir)
        return CPUT_SUCCESS;
#else
    struct stat fileAttributes;
    
    int result = stat(path.c_str(), &fileAttributes);
    if (result == -1) {
        DEBUG_ERROR(strerror(errno));
        result = CPUT_ERROR; 
    }
	
    if (S_ISDIR(fileAttributes.st_mode)) {
        return CPUT_SUCCESS;
    }
#endif
    return CPUT_ERROR;
}

// Open a file and return file pointer to it
//-----------------------------------------------------------------------------
CPUTResult CPUTFileSystem::OpenFile(const cString &fileName, FILE **ppFilePointer)
{
#ifdef CPUT_OS_ANDROID
    DEBUG_PRINT("OpenFile stubbed");
#else
    *ppFilePointer = fopen(fileName.c_str(), "r");
    
    if (*ppFilePointer == NULL) {
        return CPUT_ERROR;
    }
#endif    
    return CPUT_SUCCESS;
}

// Read the entire contents of a file and return a pointer/size to it
//-----------------------------------------------------------------------------
CPUTResult CPUTFileSystem::ReadFileContents(const cString &fileName, UINT *pSizeInBytes, void **ppData, bool bAddTerminator, bool bLoadAsBinary)
{
#ifdef CPUT_USE_ANDROID_ASSET_MANAGER
    DEBUG_PRINT("ReadFileContents(%s)", fileName.c_str());
    
    // On Android, all files are in the APK and are compressed.
    // We do not have access to the standard file system, so
    // all files need streaming from memory through the android asset manager
    AAssetManager* assetManager = CPUTWindowAndroid::GetAppState()->activity->assetManager;
    
                // For some reason we need to pass in the fully pathed filename here, rather than the relative filename
                AAsset* asset = AAssetManager_open(assetManager, fileName.c_str()/*assetFileName*/, AASSET_MODE_STREAMING);
                if (!asset)
                {
                                DEBUG_PRINT("Asset failed to load file %s", fileName.c_str());
                                return CPUT_ERROR;
                }
    
                *pSizeInBytes = AAsset_getLength(asset);
    
                // allocate buffer
                if (bAddTerminator)
                                *ppData = (void*) new char[*pSizeInBytes + 1];
                else
                                *ppData = (void*) new char[*pSizeInBytes];
                
                if (!*ppData)
                {
                                DEBUG_PRINT("Out of memory loading %s", fileName.c_str());
                                return CPUT_ERROR;
                }
    
                // read it all in
                int numBytesRead = AAsset_read(asset, *ppData, *pSizeInBytes);
                if (bAddTerminator)
                {
                                ((char *)(*ppData))[numBytesRead++] = '\0';
                                *pSizeInBytes++;
                }
                ASSERT( numBytesRead == *pSizeInBytes, _L("File read byte count mismatch.") );
   
                AAsset_close(asset);
                return CPUT_SUCCESS;
#else // CPUT_USE_ANDROID_ASSET_MANAGER
    FILE *pFile = NULL;

    DEBUG_PRINT("ReadFileContents: %s", fileName.c_str());

    if (bLoadAsBinary) {
#if defined (UNICODE) || defined(_UNICODE)
    	_wfopen_s(&pFile, fileName.c_str(), _L("rb"));
#else
    	pFile = fopen(fileName.c_str(), "rb");
#endif
    } else {
#if defined (UNICODE) || defined(_UNICODE)
    	_wfopen_s(&pFile, fileName.c_str(), _L("r"));
#else
    	pFile = fopen(fileName.c_str(), "r");
#endif
    }

    if(pFile)
    {
        // get file size
        fseek(pFile, 0, SEEK_END);
        *pSizeInBytes = ftell(pFile);
        fseek (pFile, 0, SEEK_SET);

        // allocate buffer
        if (bAddTerminator)
            *ppData = (void*) new char[*pSizeInBytes + 1];
        else
            *ppData = (void*) new char[*pSizeInBytes];
        ASSERT( *ppData, _L("Out of memory") );

        // read it all in
        UINT numBytesRead = (UINT) fread(*ppData, sizeof(char), *pSizeInBytes, pFile);
        if (bAddTerminator)
        {
            ((char *)(*ppData))[numBytesRead++] = '\0';
            *pSizeInBytes++;
        }
        ASSERT( numBytesRead == *pSizeInBytes, _L("File read byte count mismatch.") );

        // close and return
        fclose(pFile);
        return CPUT_SUCCESS;
    }

#endif // CPUT_USE_ANDROID_ASSET_MANAGER 
    // some kind of file error, translate the error code and return it
    return CPUT_ERROR;
//    return TranslateFileError(err);

}

// Open a system dialog box
//-----------------------------------------------------------------------------
CPUTResult CPUTOSServices::OpenMessageBox(cString title, cString text)
{
	assert(false);
	//::MessageBox(NULL, text.c_str(), title.c_str(), MB_OK);

    return CPUT_SUCCESS;
}
// Translate a file operation error code
//-----------------------------------------------------------------------------
CPUTResult CPUTFileSystem::TranslateFileError(int err)
{
    if(0==err)
    {
        return CPUT_SUCCESS;
    }

    // see: http://msdn.microsoft.com/en-us/library/t3ayayh1.aspx
    // for list of all error codes
    CPUTResult result = CPUT_ERROR_FILE_ERROR;

    switch(err)
    {
    case ENOENT: result = CPUT_ERROR_FILE_NOT_FOUND;                 break; // file/dir not found
    case EIO:    result = CPUT_ERROR_FILE_IO_ERROR;                  break;
    case ENXIO:  result = CPUT_ERROR_FILE_NO_SUCH_DEVICE_OR_ADDRESS; break;
    case EBADF:  result = CPUT_ERROR_FILE_BAD_FILE_NUMBER;           break;
    case ENOMEM: result = CPUT_ERROR_FILE_NOT_ENOUGH_MEMORY;         break;
    case EACCES: result = CPUT_ERROR_FILE_PERMISSION_DENIED;         break;
    case EBUSY:  result = CPUT_ERROR_FILE_DEVICE_OR_RESOURCE_BUSY;   break;
    case EEXIST: result = CPUT_ERROR_FILE_EXISTS;                    break;
    case EISDIR: result = CPUT_ERROR_FILE_IS_A_DIRECTORY;            break;
    case ENFILE: result = CPUT_ERROR_FILE_TOO_MANY_OPEN_FILES;       break;
    case EFBIG:  result = CPUT_ERROR_FILE_TOO_LARGE;                 break;
    case ENOSPC: result = CPUT_ERROR_FILE_DEVICE_FULL;               break;
    case ENAMETOOLONG: result = CPUT_ERROR_FILE_FILENAME_TOO_LONG;   break;
    default:
        // unknown file error type - assert so you can add it to the list
        ASSERT(0,_L("Unkown error code"));
    }
    return result;
}

#ifdef CPUT_GPA_INSTRUMENTATION
// Allows you to get the global/domain-wide instrumentation markers needed
// to mark events in GPA
//-----------------------------------------------------------------------------
void CPUTOSServices::GetInstrumentationPointers(__itt_domain **ppGPADomain, CPUT_GPA_INSTRUMENTATION_STRINGS eString, __itt_string_handle **ppGPAStringHandle)
{
    *ppGPADomain = mpGPADomain;
    *ppGPAStringHandle = mppGPAStringHandles[eString];
}

// Set the global/domain-wide instrumtation markers needed to mark events
// in GPA
//-----------------------------------------------------------------------------
void CPUTOSServices::SetInstrumentationPointers(__itt_domain *pGPADomain, CPUT_GPA_INSTRUMENTATION_STRINGS eString, __itt_string_handle *pGPAStringHandle)
{
    mpGPADomain = pGPADomain;
    mppGPAStringHandles[eString] = pGPAStringHandle;
}
#endif


#ifdef CPUT_USE_ANDROID_ASSET_MANAGER

CPUTFileSystem::CPUTandroidifstream::CPUTandroidifstream(const cString &fileName, std::ios_base::openmode mode) : iCPUTifstream(fileName, mode)
{
    mpAsset = NULL;
    mpAssetDir = NULL;
    mbEOF = true;
    
    // On Android, all files are in the APK and are compressed.
    // We do not have access to the standard file system, so
    // all files need streaming from memory through the android asset manager
    AAssetManager* assetManager = CPUTWindowAndroid::GetAppState()->activity->assetManager;
                mpAsset = AAssetManager_open(assetManager, fileName.c_str()/*assetFileName*/, AASSET_MODE_STREAMING);
                if (mpAsset)
                                mbEOF = false;
                return;
}
        
CPUTFileSystem::CPUTandroidifstream::~CPUTandroidifstream()
{
    if (mpAsset)
        AAsset_close(mpAsset);
    if (mpAssetDir)
        AAssetDir_close(mpAssetDir);    
}

void CPUTFileSystem::CPUTandroidifstream::read(char* s, int64_t n)
{
    if (mbEOF || !mpAsset)
        return;
    
    int numBytesRead = AAsset_read(mpAsset, s, n);
    if (numBytesRead != n)
        mbEOF = true;
}

void CPUTFileSystem::CPUTandroidifstream::close()
{
    if (mpAsset)
        AAsset_close(mpAsset);
    if (mpAssetDir)
        AAssetDir_close(mpAssetDir);    
    mpAsset = NULL;
    mpAssetDir = NULL;
}

#endif // CPUT_USE_ANDROID_ASSET_MANAGER
