// Copyright 2011 Intel Corporation
// All Rights Reserved
//
// Permission is granted to use, copy, distribute and prepare derivative works of this
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  THIS SOFTWARE IS PROVIDED "AS IS."
// INTEL SPECIFICALLY DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, AND ALL LIABILITY,
// INCLUDING CONSEQUENTIAL AND OTHER INDIRECT DAMAGES, FOR THE USE OF THIS SOFTWARE,
// INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PROPRIETARY RIGHTS, AND INCLUDING THE
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.

#ifndef H_AIOT_RESOLVE
#define H_AIOT_RESOLVE

#include "..\IGFXExtensions\IntelExtensions.hlsl"
#include "AOIT.hlsl"

float4 AOITSPResolvePS(float4 pos: SV_POSITION, float2 tex : TEX_COORD0) : SV_Target
{
	float4 outColor = float4(0, 0, 0, 1);
    uint2 pixelAddr = uint2(pos.xy);

    // display debug colour
    //return float4( 0, 1, 0, 1.0 );

	// Load control surface
	AOITCtrlSurface ctrlSurface;
	AOITLoadControlSurfaceSRV(pixelAddr, ctrlSurface);

	// Any transparent fragment contributing to this pixel?
	if (!ctrlSurface.clear) 
	{
		// Load all nodes for this pixel    
		ATSPNode nodeArray[AOIT_NODE_COUNT];
		AOITSPLoadDataSRV(pixelAddr, nodeArray);

		// Accumulate final transparent colors
		float  trans = 1;
		float3 color = 0;       
		[unroll]for(uint i = 0; i < AOIT_NODE_COUNT; i++) {
			color += trans * UnpackRGB(nodeArray[i].color);
			trans  = nodeArray[i].trans / 255;
		}
		outColor = float4(color, nodeArray[AOIT_NODE_COUNT - 1].trans / 255);
	}

    // blend accumualted transparent color with opaque background color
    return outColor;
}

[earlydepthstencil]
void AOITSPClearPS( float4 pos: SV_POSITION, float2 tex : TEX_COORD0 )
{
    uint2 pixelAddr = uint2(pos.xy);

	uint addr = AOITAddrGenUAV(pixelAddr);

	uint data = 0x1; // is clear
	gAOITSPClearMaskUAV[pixelAddr] = data;

}

#endif // H_AIOT_RESOLVE
