//--------------------------------------------------------------------------------------
// Copyright 2012 Intel Corporation
// All Rights Reserved
//
// Permission is granted to use, copy, distribute and prepare derivative works of this
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  THIS SOFTWARE IS PROVIDED "AS IS."
// INTEL SPECIFICALLY DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, AND ALL LIABILITY,
// INCLUDING CONSEQUENTIAL AND OTHER INDIRECT DAMAGES, FOR THE USE OF THIS SOFTWARE,
// INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PROPRIETARY RIGHTS, AND INCLUDING THE
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
//--------------------------------------------------------------------------------------

#include "CPUT.h"
#include "OIT.h"
#include "CPUTRenderTarget.h"
#include "CPUTTextureDX11.h"
#include "CPUTZoom.h"

extern CPUTZoomBox                 g_CPUTZoomBox;


struct AOITDebugViewConsts
{
    int a;
    int b;
    int c;
    int d;
};


void MySample::Shutdown()
{
	SAFE_RELEASE(mpResolveMaterial);
	SAFE_RELEASE(mpAOITDebugDepthMaterial);
	SAFE_RELEASE(mpAOITDebugMSAADepthMaterial);
	SAFE_RELEASE(mpAOITDebugViewMaterial);
	SAFE_RELEASE(mpAOITDebugViewConsts);

	g_CPUTZoomBox.OnShutdown();

	DeleteDebugViews();

	if(GetFeatureLevel() >= D3D_FEATURE_LEVEL_11_0)
		mPostProcess.OnShutdown();

       // Note: these two are defined in the base.  We release them because we addref them.
		SAFE_RELEASE(mpGrassA1Material);
		SAFE_RELEASE(mpGrassA2Material);
		SAFE_RELEASE(mpLeavesTreeAMaterial);
		SAFE_RELEASE(mpPRP_ShrubSmall_DM_AMaterial);
		SAFE_RELEASE(mpGlassMaterial);
		SAFE_RELEASE(mpFenceMaterial);
		SAFE_RELEASE(mpSkyBoxMaterial);

        SAFE_RELEASE(mpCamera);
		SAFE_RELEASE(mpShadowCamera);
        SAFE_RELEASE(mpConservatorySet);
		SAFE_RELEASE(mpGroundSet);
		SAFE_RELEASE(mpoutdoorPlantsSet);
		SAFE_RELEASE( mpBarrierSet );
        SAFE_DELETE( mpShadowRenderTarget );
		SAFE_DELETE( mpInternalShadowRenderTarget);

		SAFE_RELEASE(mpindoorPlantsSet);
        SAFE_DELETE( mpCameraController );
        SAFE_DELETE( mpDebugSprite);
		SAFE_DELETE( mpSkyBoxSprite );
		SAFE_DELETE( mpFSSprite );
        SAFE_DELETE( mpDebugSprite);
        SAFE_DELETE( mpMSAABackBuffer);
		SAFE_DELETE( mpMSAADepthBuffer);
        SAFE_DELETE( mpBackBuffer);
        SAFE_DELETE( mpDepthBuffer);

    CPUT_DX11::Shutdown();
}

//-----------------------------------------------------------------------------
void MySample::RegisterUI()
{
    CPUTGuiControllerDX11 *pGUI = CPUTGetGuiController();

    // create some controls

    CPUTCheckbox   *pVsync = NULL;
    pGUI->CreateCheckbox(_L("VSYNC"),    ID_VSYNC_CHECKBOX,    ID_MAIN_PANEL, &pVsync);
	pVsync->SetCheckboxState( (mSyncInterval != 0)? CPUT_CHECKBOX_CHECKED : CPUT_CHECKBOX_UNCHECKED);
 
    pGUI->CreateDropdown(_L("Alpha Blending"), ID_RENDERTYPE, ID_MAIN_PANEL, &mpRenderType);
	mpRenderType->AddSelectionItem(_L("Alpha Blending + A2C"));

	if(GetFeatureLevel() >= D3D_FEATURE_LEVEL_11_0)
		mpRenderType->AddSelectionItem(_L("DX11 AOIT"));

	if(true == mAVSMExtensionAvailable)
    {
		mpRenderType->AddSelectionItem(_L("Pixel Synchronization OIT"));
	}

	if(true == mAVSMExtensionAvailable)
    {
		pGUI->CreateDropdown(_L("2 Nodes"), ID_RENDERTYPE, ID_MAIN_PANEL, &mpNodeCount);
		mpNodeCount->AddSelectionItem(_L("4 Nodes"));
		mpNodeCount->AddSelectionItem(_L("8 Nodes"));

		mpNodeCount->SetEnable(false);
	}

#ifdef AOIT_METRICS
    pGUI->CreateCheckbox(_L("Show Stats"), ID_ENABLE_STATS, ID_MAIN_PANEL, &mpShowStats);
	mpShowStats->SetCheckboxState(CPUT_CHECKBOX_CHECKED);
#endif
    pGUI->CreateCheckbox(_L("Write Foliage to Depth"), ID_ENABLE_STATS, ID_MAIN_PANEL, &mpSortFoliage);
	mpSortFoliage->SetCheckboxState(CPUT_CHECKBOX_UNCHECKED);

    pGUI->CreateCheckbox(_L("DepthBuffer"), ID_DEPTH, ID_MAIN_PANEL, &mpShowDepth);
	pGUI->CreateCheckbox(_L("Enable Pixel Sync"), ID_RENDERTRANSPARENT, ID_MAIN_PANEL, &mbPixelSync);
	mbPixelSync->SetCheckboxState(CPUT_CHECKBOX_CHECKED);

	if(GetFeatureLevel() >= D3D_FEATURE_LEVEL_11_0)
	{
		pGUI->CreateCheckbox(_L("Enable Zoom"), ID_RESOLVE, ID_MAIN_PANEL, &mpZoomBox);
		mpZoomBox->SetCheckboxState(CPUT_CHECKBOX_UNCHECKED);
	}
	else
    {
        mpZoomBox = NULL;
	}

	if(true == mAVSMExtensionAvailable)
    {
		pGUI->CreateCheckbox(_L("Show Pixel Sync debug view"), ID_DEBUG_DISPLAY, ID_MAIN_PANEL, &mpShowDebugView);
		mpShowDebugView->SetEnable(false);
	}
#ifdef AOIT_METRICS
	pGUI->CreateText( _L("(ms):"), ID_AOITT, ID_MAIN_PANEL,&mpPPT);
    mpPPT->SetAutoArranged(false);
	mpPPT->SetPosition(5,50);
	mpPPT->SetVisibility(CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() && mpShowStats->IsEnabled());

	pGUI->CreateText( _L("(ms):"), ID_AOITT, ID_MAIN_PANEL,&mpAOITT);
    mpAOITT->SetAutoArranged(false);
	mpAOITT->SetPosition(5,65);
	mpAOITT->SetVisibility(CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() && mpShowStats->IsEnabled());

	pGUI->CreateText( _L("(ms):"), ID_RT, ID_MAIN_PANEL,&mpRT);
    mpRT->SetAutoArranged(false);
	mpRT->SetPosition(5,80);
	mpRT->SetVisibility(CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() && mpShowStats->IsEnabled());

	pGUI->CreateText( _L("(ms):"), ID_TT, ID_MAIN_PANEL,&mpTT);
    mpTT->SetAutoArranged(false);
	mpTT->SetPosition(5,95);
	mpTT->SetVisibility(CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() && mpShowStats->IsEnabled());
#endif
}


void MySample::AddGlobalProperties()
{
    CPUTMaterial::mGlobalProperties.AddValue( _L("gFragmentListNodesUAV"), _L("$gFragmentListNodesUAV") );
    CPUTMaterial::mGlobalProperties.AddValue( _L("gFragmentListFirstNodeAddressUAV"), _L("$gFragmentListFirstNodeAddressUAV") );
    CPUTMaterial::mGlobalProperties.AddValue( _L("gAOITSPClearMaskUAV"), _L("$gAOITSPClearMaskUAV") );
    CPUTMaterial::mGlobalProperties.AddValue( _L("gAOITSPDepthDataUAV"), _L("$gAOITSPDepthDataUAV") );
    CPUTMaterial::mGlobalProperties.AddValue( _L("gAOITSPColorDataUAV"), _L("$gAOITSPColorDataUAV") );
    CPUTMaterial::mGlobalProperties.AddValue( _L("g8AOITSPDepthDataUAV"), _L("$g8AOITSPDepthDataUAV") );
    CPUTMaterial::mGlobalProperties.AddValue( _L("g8AOITSPColorDataUAV"), _L("$g8AOITSPColorDataUAV") );
    CPUTMaterial::mGlobalProperties.AddValue( _L("g_IntelExt"), _L("$g_IntelExt") );

	CPUTMaterial::mGlobalProperties.AddValue( _L("cbAOITDebugViewConsts"), _L("$cbAOITDebugViewConsts") );
    CPUTMaterial::mGlobalProperties.AddValue( _L("FL_Constants"), _L("$FL_Constants") );
}

void MySample::CreateDebugViews()
{
    CPUTAssetLibrary *pAssetLibrary = CPUTAssetLibrary::GetAssetLibrary();

    // Create our constant buffer.

    D3D11_BUFFER_DESC bd = {0};
    bd.ByteWidth = sizeof(AOITDebugViewConsts);
    bd.BindFlags = D3D11_BIND_CONSTANT_BUFFER;
    bd.Usage = D3D11_USAGE_DYNAMIC;
    bd.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;

    ID3D11Buffer * pOurConstants;
    HRESULT hr = (CPUT_DX11::GetDevice())->CreateBuffer( &bd, NULL, &pOurConstants );

    cString name = _L("$cbAOITDebugViewConsts");
    mpAOITDebugViewConsts = new CPUTBufferDX11( name, pOurConstants );

    CPUTAssetLibrary::GetAssetLibrary()->AddConstantBuffer( name, _L(""), _L(""), mpAOITDebugViewConsts );
    SAFE_RELEASE(pOurConstants); // We're done with it.  The CPUTBuffer now owns it.
    //mGlobalProperties

    cString ExecutableDirectory;
	CPUTFileSystem::GetExecutableDirectory(&ExecutableDirectory);

   pAssetLibrary->SetMediaDirectoryName(ExecutableDirectory +     _L("..\\..\\..\\Media\\DebugAssets\\") );
   mpAOITDebugViewMaterial = pAssetLibrary->GetMaterial( L"AOITDebugView" );
   mpAOITDebugDepthMaterial = pAssetLibrary->GetMaterial( L"AOITDebugDepth" );
   mpAOITDebugMSAADepthMaterial = pAssetLibrary->GetMaterial( L"AOITDebugMSAADepth" );
   mpResolveMaterial = pAssetLibrary->GetMaterial( L"ResolveTarget" );

   mpDebugSprite = CPUTSprite::CreateSprite(0.5f, 0.5f, 0.5f, 0.5f, _L("Sprite") );


   mpDepthSprite = CPUTSprite::CreateSprite(-1.0f, 0.5f, 0.5f, 0.5f, _L("Sprite") );

   mpResolveSprite = CPUTSprite::CreateSprite(-1.0f, -1.0f, 2.0f, 2.0f, _L("ResolveTarget") );

}


void MySample::DisplayDebugViews(CPUTRenderParametersDX &renderParams)
{
	CPUTAssetLibrary *pAssetLibrary = CPUTAssetLibrary::GetAssetLibrary();


	if(true == mAVSMExtensionAvailable)
	{
		ID3D11Buffer *pBuffer = mpAOITDebugViewConsts->GetNativeBuffer();
		// update parameters of constant buffer
		D3D11_MAPPED_SUBRESOURCE mapInfo;
		mpContext->Map( pBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mapInfo );
		{
			AOITDebugViewConsts & consts = *((AOITDebugViewConsts*)mapInfo.pData);
            
			// for future use
			consts.a = 0;
			consts.b = 8;
			consts.c = 0;
			consts.d = 2;
			mpContext->Unmap(pBuffer,0);
		}
		mpContext->VSSetConstantBuffers( 4, 1, &pBuffer );
		mpContext->PSSetConstantBuffers( 4, 1, &pBuffer );
		mpContext->CSSetConstantBuffers( 4, 1, &pBuffer );
		if( (mpAOITDebugViewConsts != NULL) && (mpShowDebugView) && (CPUT_CHECKBOX_CHECKED == mpShowDebugView->GetCheckboxState()) && (mpShowDebugView->IsEnabled()))
		{

			mpDebugSprite->DrawSprite( renderParams, *mpAOITDebugViewMaterial );
			ID3D11ShaderResourceView* nullViews[32] = {0};
			mpContext->PSSetShaderResources(0, 32, nullViews);
		}
	}
	if( (mpAOITDebugViewConsts != NULL) && (CPUT_CHECKBOX_CHECKED == mpShowDepth->GetCheckboxState()) )
	{
		ID3D11RenderTargetView* pSwapChainRTV;		// render target view retrieved at InitFrameRender
		ID3D11DepthStencilView* pDSView;			// depth stencil view retried at InitFrameRender
		cString SelectedItem;
		mpRenderType->GetSelectedItem(SelectedItem);

		mpContext->OMGetRenderTargets(1, &(pSwapChainRTV), &pDSView);

		mpContext->OMSetRenderTargets(1, &pSwapChainRTV, NULL);
		
		if(_L("Alpha Blending + A2C") == SelectedItem)
		{
			mpDepthSprite->DrawSprite( renderParams, *mpAOITDebugMSAADepthMaterial );
		}
		else
		{
			mpDepthSprite->DrawSprite( renderParams, *mpAOITDebugDepthMaterial );
		}
		ID3D11ShaderResourceView* nullViews[32] = {0};
		mpContext->PSSetShaderResources(0, 32, nullViews);
		mpContext->OMSetRenderTargets(1, &pSwapChainRTV, pDSView);
		SAFE_RELEASE(pSwapChainRTV);
		SAFE_RELEASE(pDSView);
	}
}


void MySample::DeleteDebugViews(  )
{
	SAFE_DELETE( mpDebugSprite);
	SAFE_DELETE( mpDepthSprite);
	SAFE_DELETE( mpResolveSprite);
}


void MySample::RenderText( )
{
	FrameStats Stats;
    cString SelectedItem;
#ifdef AOIT_METRICS
    if( CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() && mpShowStats->IsEnabled() )
    {
        Stats.AOITCreationTime	= (float)(mGPUTimerAOITCreation.GetAvgTime() * 1000.0);
        Stats.ResolveTime       = (float)(mGPUTimerResolve.GetAvgTime()* 1000.0);
        Stats.TotalTime         = (float)(mGPUTimerAll.GetAvgTime() * 1000.0);
        Stats.PrePassTime         = (float)(mGPUTimerPrePass.GetAvgTime() * 1000.0);
    }
    else
    {
        memset( &Stats, 0, sizeof(Stats) );
    }

    wchar_t wcstring[CPUT_MAX_STRING_LENGTH];
    swprintf_s(wcstring, CPUT_MAX_STRING_LENGTH, _L("Transparent Render Time (ms): %.2f"), Stats.AOITCreationTime);
	mpAOITT->SetText(wcstring);

	swprintf_s(wcstring, CPUT_MAX_STRING_LENGTH, _L("ResolveTime time (ms): %.2f"), Stats.ResolveTime);
	mpRT->SetText(wcstring);

	swprintf_s(wcstring, CPUT_MAX_STRING_LENGTH, _L("Solid Geometry Render time (ms): %.2f"), Stats.PrePassTime);
	mpPPT->SetText(wcstring);

	
	mpRenderType->GetSelectedItem(SelectedItem);

	if((_L("DX11 AOIT") == SelectedItem) || (_L("Pixel Synchronization OIT") == SelectedItem))
	{
		mpRT->SetVisibility(CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() && mpShowStats->IsEnabled());
		mpAOITT->SetVisibility(CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() && mpShowStats->IsEnabled());
	}
	else
	{
		mpRT->SetVisibility(false);
		mpAOITT->SetVisibility(false);
	}

	mpRT->SetVisibility(CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() && mpShowStats->IsEnabled());
	mpPPT->SetVisibility(CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() && mpShowStats->IsEnabled());

    swprintf_s(wcstring, CPUT_MAX_STRING_LENGTH, _L("TotalTime (ms): %.2f"), Stats.TotalTime);
	mpTT->SetText(wcstring);
	mpTT->SetVisibility(CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() && mpShowStats->IsEnabled());
#endif
}

