//--------------------------------------------------------------------------------------
// Copyright 2013 Intel Corporation
// All Rights Reserved
//
// Permission is granted to use, copy, distribute and prepare derivative works of this
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  THIS SOFTWARE IS PROVIDED "AS IS."
// INTEL SPECIFICALLY DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, AND ALL LIABILITY,
// INCLUDING CONSEQUENTIAL AND OTHER INDIRECT DAMAGES, FOR THE USE OF THIS SOFTWARE,
// INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PROPRIETARY RIGHTS, AND INCLUDING THE
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
//--------------------------------------------------------------------------------------
#include "OIT.h"
#include "CPUTRenderTarget.h"
#include <CPUTFont.h>
#include "CPUTBufferDX11.h"
#include <DXGIDebug.h>
#include "CPUTTextureDX11.h"
#include ".\IGFXExtensions\IGFXExtensionsHelper.h"
#include "CPUTZoom.h"

const UINT SHADOW_WIDTH_HEIGHT = 8192;
// set file to open
cString g_OpenFilePath;
cString g_OpenShaderPath;
cString g_OpenFileName;
std::string g_OpenSceneFileName;
CPUTZoomBox                 g_CPUTZoomBox;


struct DrawNode{

	DrawNode(CPUTModelDX11* pModel, CPUTMeshDX11* pMesh, CPUTMaterialEffect *pMaterial, ID3D11InputLayout *mpInputLayout)
	{
		m_pMesh = pMesh;
		m_pModel = pModel;
		m_pMaterial = pMaterial;
		m_mpInputLayout = mpInputLayout;
	}

	CPUTModelDX11* m_pModel;
	CPUTMeshDX11* m_pMesh;
	CPUTMaterialEffect *m_pMaterial;
	ID3D11InputLayout *m_mpInputLayout;
};

static std::vector<DrawNode> m_pMeshNode;
static std::vector<DrawNode> m_pTransparentNode;



void BeginDrawLists()
{
	m_pMeshNode.clear();
	m_pTransparentNode.clear();
}


void AddModel(CPUTModelDX11* pModel,CPUTMeshDX11* pMesh, CPUTMaterialEffect *pMaterial, ID3D11InputLayout *mpInputLayout, bool Transparent)
{
	if(Transparent)
	{
   		m_pTransparentNode.push_back(DrawNode(pModel, pMesh, pMaterial, mpInputLayout ));	
	}
	else
	{
   		m_pMeshNode.push_back(DrawNode(pModel, pMesh, pMaterial, mpInputLayout ));	
	}
}

void DrawSolidLists(CPUTRenderParameters &renderParams)
{
	CPUTModelDX11* pCurrentModel = NULL;

	for(unsigned int x=0; x< m_pMeshNode.size(); x++)
	{
		// Update the model's constant buffer.
		// Note that the materials reference this, so we need to do it only once for all of the model's meshes.
		if(pCurrentModel != m_pMeshNode[x].m_pModel)
		{
			pCurrentModel = m_pMeshNode[x].m_pModel;
			pCurrentModel->UpdateShaderConstants(renderParams);
		}

        m_pMeshNode[x].m_pMaterial->SetRenderStates(renderParams);

		m_pMeshNode[x].m_pMesh->Draw(renderParams, m_pMeshNode[x].m_mpInputLayout);
	}
}

void DrawTransparentLists(CPUTRenderParameters &renderParams, bool RenderTransparent)
{
	CPUTModelDX11* pCurrentModel = NULL;

	if(RenderTransparent)
	{
		for(unsigned int x=0; x< m_pTransparentNode.size(); x++)
		{
			// Update the model's constant buffer.
			// Note that the materials reference this, so we need to do it only once for all of the model's meshes.
			if(pCurrentModel != m_pTransparentNode[x].m_pModel)
			{
				pCurrentModel = m_pTransparentNode[x].m_pModel;
				pCurrentModel->UpdateShaderConstants(renderParams);
			}

			m_pTransparentNode[x].m_pMaterial->SetRenderStates(renderParams);

			m_pTransparentNode[x].m_pMesh->Draw(renderParams, m_pTransparentNode[x].m_mpInputLayout);
		}
	}
}


bool DrawModelCallBack(CPUTModel* pModel, CPUTRenderParameters &renderParams, CPUTMesh* pMesh, CPUTMaterialEffect* pMaterial,CPUTMaterialEffect* pOriginalMaterial, void* pInputLayout)
{
	ID3D11InputLayout *pLayout = (ID3D11InputLayout *)pInputLayout;
	pMaterial->SetRenderStates(renderParams);

	AddModel((CPUTModelDX11*)pModel, (CPUTMeshDX11*)pMesh, (CPUTMaterialEffectDX11*)pMaterial, pLayout,((CPUTMaterialEffectDX11*)pOriginalMaterial)->mLayer == CPUT_LAYER_TRANSPARENT);

	return true;
}

// Application entry point.  Execution begins here.
//-----------------------------------------------------------------------------
int WINAPI wWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow )
{
    // Prevent unused parameter compiler warnings
    UNREFERENCED_PARAMETER(hInstance);
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(nCmdShow);

#ifdef DEBUG
    // tell VS to report leaks at any exit of the program
    _CrtSetDbgFlag ( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
	//http://msdn.microsoft.com/en-us/library/x98tx3cf%28v=vs.100%29.aspx
	//Add a watch for {,,msvcr100d.dll}_crtBreakAlloc to the watch window
	//Set the value of the watch to the memory allocation number reported by your sample at exit.
	//Note that the msvcr100d.dll is for MSVC2010.  Other versions of MSVC use different versions of this dl; youll need to specify the appropriate version.

#endif

    CPUTResult result=CPUT_SUCCESS;
    int returnCode=0;

	CPUTModel::SetDrawModelCallBack(DrawModelCallBack);

    // create an instance of my sample
    MySample *pSample = new MySample();

    // window and device parameters
    CPUTWindowCreationParams params;
    params.deviceParams.refreshRate         = 0;
    params.deviceParams.swapChainBufferCount= 1;
    params.deviceParams.swapChainFormat     = DXGI_FORMAT_R8G8B8A8_UNORM_SRGB;
    params.deviceParams.swapChainUsage      = DXGI_USAGE_RENDER_TARGET_OUTPUT | DXGI_USAGE_SHADER_INPUT;

    // parse out the parameter settings or reset them to defaults if not specified
    std::string AssetFilename;
    g_OpenSceneFileName = "../../../Media/defaultscene.scene";
    char *pCommandLine = ws2s(lpCmdLine);
    std::string CommandLine(pCommandLine);
    pSample->CPUTParseCommandLine(CommandLine, &params, &AssetFilename, &g_OpenSceneFileName);
    delete pCommandLine;

    // parse out the filename of the .set file to open (if one was given)
    if(AssetFilename.size())
    {
        // strip off the target .set file, and parse it's full path
        cString PathAndFilename, Drive, Dir, Ext;  
        g_OpenFilePath.clear();
        std::wstring wsAssetFileName = s2ws(AssetFilename.c_str());
        
        // resolve full path, and check to see if the file actually exists
        CPUTFileSystem::ResolveAbsolutePathAndFilename(wsAssetFileName, &PathAndFilename);
        result = CPUTFileSystem::DoesFileExist(PathAndFilename);
        if(CPUTFAILED(result))
        {
            cString ErrorMessage = _L("File specified in the command line does not exist: \n")+PathAndFilename;
            #ifndef DEBUG
            DEBUGMESSAGEBOX(_L("Error loading command line file"), ErrorMessage);
            #else
            ASSERT(false, ErrorMessage);
            #endif
        }
        
        // now parse through the path and removing the trailing \\Asset\\ directory specification
        CPUTFileSystem::SplitPathAndFilename(PathAndFilename, &Drive, &Dir, &g_OpenFileName, &Ext);
        // strip off any trailing \\ 
        size_t index=Dir.size()-1;
        if(Dir[index]=='\\' || Dir[index]=='/')
        {
            index--;
        }

        // strip off \\Asset
        for(size_t ii=index; ii>=0; ii--)
        {
            if(Dir[ii]=='\\' || Dir[ii]=='/')
            {
                Dir = Dir.substr(0, ii+1); 
                g_OpenFilePath = Drive+Dir; 
                index=ii;
                break;
            }
        }        
        
        // strip off \\<setname> 
        for(size_t ii=index; ii>=0; ii--)
        {
            if(Dir[ii]=='\\' || Dir[ii]=='/')
            {
                Dir = Dir.substr(0, ii+1); 
                g_OpenShaderPath = Drive + Dir + _L("\\Shader\\");
                break;
            }
        }
    }

    // create the window and device context
    result = pSample->CPUTCreateWindowAndContext(_L("CPUTWindow DirectX 11"), params);
    ASSERT( CPUTSUCCESS(result), _L("CPUT Error creating window and context.") );
    
    // start the main message loop
    returnCode = pSample->CPUTMessageLoop();

    pSample->DeviceShutdown();

    // cleanup resources
    SAFE_DELETE(pSample);

#if 0
    //
    // Report on unreleased DX objects. Requires the D3D11_CREATE_DEVICE_DEBUG flag be set.
    //
    typedef HRESULT(__stdcall *fPtrDXGIGetDebugInterface)(const IID&, void**);
    HMODULE hMod = GetModuleHandle(L"Dxgidebug.dll");
    fPtrDXGIGetDebugInterface DXGIGetDebugInterface = (fPtrDXGIGetDebugInterface)GetProcAddress(hMod, "DXGIGetDebugInterface"); 
 
    IDXGIDebug *pDebugInterface;
    DXGIGetDebugInterface(__uuidof(IDXGIDebug), (void**)&pDebugInterface); 

    pDebugInterface->ReportLiveObjects(DXGI_DEBUG_D3D11, DXGI_DEBUG_RLO_ALL);
#endif

    return returnCode;
}

//-----------------------------------------------------------------------------
void MySample::CreateCameras(UINT width, UINT height)
{
    CPUTAssetLibrary *pAssetLibrary = CPUTAssetLibrary::GetAssetLibrary();

    // If no cameras were created from the model sets then create a default simple camera
    if( mpConservatorySet && mpConservatorySet->GetCameraCount() )
    {
        mpCamera = mpConservatorySet->GetFirstCamera();
        mpCamera->AddRef(); // TODO: why?  Shouldn't need this.
    } 
	else
    {
        mpCamera = new CPUTCamera();
        pAssetLibrary->AddCamera( _L("Camera"),_L(""), _L(""),  mpCamera );

        mpCamera->SetPosition( 26.00f, 3.00f, 25.00f );
		mpCamera->LookAt(11.00f,5.33f,15.00f);
        // Set the projection matrix for all of the cameras to match our window.
        // TODO: this should really be a viewport matrix.  Otherwise, all cameras will have the same FOV and aspect ratio, etc instead of just viewport dimensions.
        mpCamera->SetAspectRatio(((float)width)/((float)height));
    }

    const float defaultFOVRadians = 3.14f/2.0f;
    mpCamera->SetFov(defaultFOVRadians); // TODO: Fix converter's FOV bug (Maya generates cameras for which fbx reports garbage for fov)
    mpCamera->SetFarPlaneDistance(100000.0f);
    mpCamera->Update();


    // Position and orient the shadow camera so that it sees the whole scene.
    // Set the near and far planes so that the frustum contains the whole scene.
    // Note that if we are allowed to move the shadow camera or the scene, this may no longer be true.
    // Consider updating the shadow camera when it (or the scene) moves.
    // Treat bounding box half as radius for an approximate bounding sphere.
    // The tightest-fitting sphere might be smaller, but this is close enough for placing our shadow camera.
    // Set up the shadow camera (a camera that sees what the light sees)
    float3 lookAtPoint(0.0f, 0.0f, 0.0f);
    float3 half(1.0f, 1.0f, 1.0f);
    if( mpBarrierSet )
    {
        mpBarrierSet->GetBoundingBox( &lookAtPoint, &half );
    }

    float  length = half.length();
    mpShadowCamera = new CPUTCamera( CPUT_ORTHOGRAPHIC );
    mpShadowCamera->SetAspectRatio(1.0f);
    mpShadowCamera->SetNearPlaneDistance(1.0f);
    mpShadowCamera->SetFarPlaneDistance(2.0f*length + 1.0f);
    mpShadowCamera->SetPosition( lookAtPoint - float3(0, -1, 1) * length );
    mpShadowCamera->LookAt( lookAtPoint.x,lookAtPoint.y,lookAtPoint.z );
    mpShadowCamera->SetWidth( length*2);
    mpShadowCamera->SetHeight(length*2);
    mpShadowCamera->Update();

    mpCameraController = new CPUTCameraControllerFPS();
    mpCameraController->SetCamera(mpCamera);
    mpCameraController->SetLookSpeed(0.004f);
    mpCameraController->SetMoveSpeed(20.0f);

}
//-----------------------------------------------------------------------------
void MySample::Create()
{    

	///////////////////////////////////////////////////////////////////////////////
	// Intel extension support check
	///////////////////////////////////////////////////////////////////////////////
    HRESULT hr = IGFX::Init( this->mpD3dDevice );
	if ( FAILED(hr) )
	{
		printf("Failed extensions initialization: incorrect vendor or device.\n\n");
	}

    // detect the available extensions
    IGFX::Extensions extensions = IGFX::getAvailableExtensions( this->mpD3dDevice );

    // report any problems
    if ( extensions.PixelShaderOrdering ) 
    {
         mAVSMExtensionAvailable = true;
    }
    else
    {
        CPUTOSServices::OpenMessageBox(_L("UAV serialization extension not found"), _L("Your system or driver does not support UAV serialization extension.  Please update your driver or run on a system that supports the required extension."));
    }

    // Call ResizeWindow() because it creates some resources that our blur material needs (e.g., the back buffer)
    int width, height;
    mpWindow->GetClientDimensions(&width, &height);


    CPUTRenderStateBlockDX11 *pBlock = new CPUTRenderStateBlockDX11();
    CPUTRenderStateDX11 *pStates = pBlock->GetState();

    CPUTAssetLibrary *pAssetLibrary = CPUTAssetLibrary::GetAssetLibrary();

    cString ExecutableDirectory;
    cString mediaDirectory;
    CPUTFileSystem::GetExecutableDirectory(&ExecutableDirectory);
    CPUTFileSystem::ResolveAbsolutePathAndFilename(ExecutableDirectory + _L("../../../Media/"), &mediaDirectory);

    pAssetLibrary->SetMediaDirectoryName(mediaDirectory);
    pAssetLibrary->SetSystemDirectoryName(mediaDirectory + _L("System/"));

    CPUTGuiControllerDX11 *pGUI = CPUTGetGuiController();
    pAssetLibrary->SetAssetDirectoryName(mediaDirectory + _L("gui_assets/"));
    pGUI->Initialize(mpContext, mediaDirectory);
    pGUI->SetCallback(this);
    pGUI->SetWindow(mpWindow);

    CPUTFont *pFont = CPUTFont::CreateFont(mediaDirectory + _L("System/Font/"), _L("arial_16.fnt"));
    pGUI->SetFont(pFont);

    // Create some controls
    //
    // Create some controls
    //    
    CPUTButton     *pButton = NULL;
    pGUI->CreateButton(_L("Fullscreen"), ID_FULLSCREEN_BUTTON, ID_MAIN_PANEL, &pButton);

    RegisterUI();

    UpdateEnableStatsCheckbox();


    

    // Add our programatic (and global) material parameters
    CPUTMaterial::mGlobalProperties.AddValue( _L("cbPerFrameValues"), _L("$cbPerFrameValues") );
    CPUTMaterial::mGlobalProperties.AddValue( _L("cbPerModelValues"), _L("$cbPerModelValues") );
    CPUTMaterial::mGlobalProperties.AddValue( _L("_Shadow"), _L("$shadow_depth") );
    CPUTMaterial::mGlobalProperties.AddValue( _L("_InternalShadow"), _L("$internalshadow") );
    CPUTMaterial::mGlobalProperties.AddValue( _L("OffscreenColorBuffer"), _L("$OffscreenColorBuffer") );

	AddGlobalProperties();

	if(GetFeatureLevel() >= D3D_FEATURE_LEVEL_11_0)
		mPostProcess.OnCreate(mpD3dDevice, width, height, mpContext, mpSwapChain);

    // load shadow casting material+sprite object
    mpShadowRenderTarget = new CPUTRenderTargetDepth();
    mpShadowRenderTarget->CreateRenderTarget( cString(_L("$shadow_depth")), SHADOW_WIDTH_HEIGHT, SHADOW_WIDTH_HEIGHT, DXGI_FORMAT_D32_FLOAT );
    mpInternalShadowRenderTarget = new CPUTRenderTargetDepth();
    mpInternalShadowRenderTarget->CreateRenderTarget( cString(_L("$internalshadow")), SHADOW_WIDTH_HEIGHT, SHADOW_WIDTH_HEIGHT, DXGI_FORMAT_D32_FLOAT );


    mpBackBuffer = new CPUTRenderTargetColor;
    mpBackBuffer->CreateRenderTarget( cString(_L("$OffscreenColorBuffer")), width, height, DXGI_FORMAT_R8G8B8A8_UNORM_SRGB );
    mpDepthBuffer = new CPUTRenderTargetDepth;
    mpDepthBuffer->CreateRenderTarget( cString(_L("$OffscreenDepthBuffer")), width, height, DXGI_FORMAT_D32_FLOAT );



	g_CPUTZoomBox.OnCreate(mpD3dDevice, mpContext, mpSwapChain);

	const UINT MSAA_COUNT  = 4;

	mpMSAABackBuffer = new CPUTRenderTargetColor();
	mpMSAABackBuffer->CreateRenderTarget(cString(_L("$MSAAColorBuffer")), width, height, DXGI_FORMAT_R8G8B8A8_UNORM_SRGB, MSAA_COUNT);

	mpMSAADepthBuffer = new CPUTRenderTargetDepth();
	mpMSAADepthBuffer->CreateRenderTarget(cString(_L("$MSAADepthBuffer")), width, height, DXGI_FORMAT_D32_FLOAT, MSAA_COUNT);

    // Override default sampler desc for our default shadowing sampler
    pStates->SamplerDesc[1].Filter         = D3D11_FILTER_COMPARISON_MIN_MAG_LINEAR_MIP_POINT;
    pStates->SamplerDesc[1].AddressU       = D3D11_TEXTURE_ADDRESS_BORDER;
    pStates->SamplerDesc[1].AddressV       = D3D11_TEXTURE_ADDRESS_BORDER;
    pStates->SamplerDesc[1].ComparisonFunc = D3D11_COMPARISON_GREATER;
    pBlock->CreateNativeResources();
    pAssetLibrary->AddRenderStateBlock( _L("$DefaultRenderStates"), _L(""), _L(""), pBlock );
    pBlock->Release(); // We're done with it.  The library owns it now.

    // ***************************
    // Render the terrain to our height field texture.
    // TODO: How much of this memory can we reclaim after this step?
    // Can theoretically just release.  But, AssetLibrary holds references too.
    // ***************************

	ResizeWindow(width, height);

    pAssetLibrary->SetMediaDirectoryName( ExecutableDirectory +   _L("..\\..\\..\\Media\\conservatory_01\\") );
    mpConservatorySet = pAssetLibrary->GetAssetSet( _L("conservatory_01") );
  	mpGlassMaterial = pAssetLibrary->GetMaterial( _L("Glass1") );
	mpFenceMaterial = pAssetLibrary->GetMaterial( _L("Fence") );


	pAssetLibrary->SetMediaDirectoryName( ExecutableDirectory +    _L("..\\..\\..\\Media\\ground\\") );
    mpGroundSet = pAssetLibrary->GetAssetSet( _L("ground") );

   
	pAssetLibrary->SetMediaDirectoryName(  ExecutableDirectory +   _L("..\\..\\..\\Media\\outdoorPlants_01\\") );
    mpoutdoorPlantsSet = pAssetLibrary->GetAssetSet( _L("outdoorPlants_01") );	
	mpGrassA1Material = pAssetLibrary->GetMaterial( _L("grassA1") );
	mpGrassA2Material = pAssetLibrary->GetMaterial( _L("grassA2") );

    pAssetLibrary->SetMediaDirectoryName( ExecutableDirectory +    _L("..\\..\\..\\Media\\indoorPlants_01\\") );
    mpindoorPlantsSet = pAssetLibrary->GetAssetSet( _L("indoorPlants_01") );
	
	mpLeavesTreeAMaterial = pAssetLibrary->GetMaterial( _L("leavesTreeA") );
	mpPRP_ShrubSmall_DM_AMaterial = pAssetLibrary->GetMaterial( _L("PRP_ShrubSmall_DM_A") );
    
	pAssetLibrary->SetMediaDirectoryName( ExecutableDirectory +    _L("..\\..\\..\\Media\\barrier_01\\") );
	mpBarrierSet = pAssetLibrary->GetAssetSet( _L("barrier_01") );

    pAssetLibrary->SetMediaDirectoryName( ExecutableDirectory +    _L("..\\..\\..\\Media\\"));
    mpSkyBoxMaterial = pAssetLibrary->GetMaterial( L"SkyBox11" );

	mpSkyBoxSprite = CPUTSprite::CreateSprite( -1.0f, -1.0f, 2.0f, 2.0f, _L("SkyBox11"));

	mpFSSprite = CPUTSprite::CreateSprite( -1.0f, -1.0f, 2.0f, 2.0f, _L("FSSprite"));

	CreateDebugViews();
    
   CreateCameras(width, height);


}

//-----------------------------------------------------------------------------
void MySample::Update(double deltaSeconds)
{
    mpCameraController->Update((float)deltaSeconds);
    cString SelectedItem;

	mpRenderType->GetSelectedItem(SelectedItem);
    if( mpNodeCount != NULL )
	    mpNodeCount->SetEnable(_L("Pixel Synchronization OIT") == SelectedItem);
    
	if( mpShowDebugView != NULL )
	    mpShowDebugView->SetEnable(_L("Pixel Synchronization OIT") == SelectedItem);


	if( mpSortFoliage != NULL )
	    mpSortFoliage->SetEnable(_L("Alpha Blending") == SelectedItem);


}

// Handle keyboard events
//-----------------------------------------------------------------------------
CPUTEventHandledCode MySample::HandleKeyboardEvent(CPUTKey key, CPUTKeyState state)
{
    static bool panelToggle = false;
    CPUTEventHandledCode    handled = CPUT_EVENT_UNHANDLED;
    cString fileName;
    CPUTGuiControllerDX11*  pGUI = CPUTGetGuiController();

    switch(key)
    {
    case KEY_F1:
        panelToggle = !panelToggle;
        if(panelToggle)
        {
            pGUI->SetActivePanel(ID_SECONDARY_PANEL);
        }
        else
        {
            pGUI->SetActivePanel(ID_MAIN_PANEL);
        }
        handled = CPUT_EVENT_HANDLED;
        break;
    case KEY_L:
        {
            static int cameraObjectIndex = 0;
            CPUTRenderNode *pCameraList[] = { mpCamera, mpShadowCamera };
            cameraObjectIndex = (++cameraObjectIndex) % (sizeof(pCameraList)/sizeof(*pCameraList));
            CPUTRenderNode *pCamera = pCameraList[cameraObjectIndex];
            mpCameraController->SetCamera( pCamera );
        }
        handled = CPUT_EVENT_HANDLED;
        break;
    case KEY_ESCAPE:
        handled = CPUT_EVENT_HANDLED;
        ShutdownAndDestroy();
        break;
    }
    // pass it to the camera controller
    if(handled == CPUT_EVENT_UNHANDLED)
    {
        handled = mpCameraController->HandleKeyboardEvent(key, state);
    }

    return handled;
}


// Handle mouse events
//-----------------------------------------------------------------------------
CPUTEventHandledCode MySample::HandleMouseEvent(int x, int y, int wheel, CPUTMouseState state, CPUTEventID message)
{
	if(state & CPUT_MOUSE_RIGHT_DOWN)
	{
		FLOAT zoomx =  x / (FLOAT) m_ScreenWidth;
		FLOAT zoomy =  y / (FLOAT) m_ScreenHeight;

		g_CPUTZoomBox.SetZoomCenterPosition((float)zoomx, (float)zoomy);
	}

    if( mpCameraController )
    {
        return mpCameraController->HandleMouseEvent(x, y, wheel, state, message);
    }
    return CPUT_EVENT_UNHANDLED;
}

void MySample::HandleGUIElementEvent( CPUTEventID Event, CPUTControlID ControlID, CPUTGUIElement *pElement )
{
}
// Handle any control callback events
//-----------------------------------------------------------------------------
void MySample::HandleCallbackEvent( CPUTEventID Event, CPUTControlID ControlID, CPUTControl *pControl )
{
    UNREFERENCED_PARAMETER(Event);
    UNREFERENCED_PARAMETER(pControl);
    CPUTCheckboxState checkboxState;

    switch(ControlID)
    {
    case ID_FULLSCREEN_BUTTON:
        CPUTToggleFullScreenMode();
        break;
    case ID_VSYNC_CHECKBOX:
        checkboxState = ((CPUTCheckbox*)pControl)->GetCheckboxState();
        mSyncInterval = (checkboxState == CPUT_CHECKBOX_CHECKED);
        UpdateEnableStatsCheckbox();
        break;
    case ID_RENDERTYPE:
        UpdateEnableStatsCheckbox();
		break;
	case ID_DEBUG_DISPLAY:
        UpdateEnableStatsCheckbox();
        break;
    default:
        break;
    }
}

//-----------------------------------------------------------------------------
void MySample::ResizeWindow(UINT width, UINT height)
{
    CPUTAssetLibrary *pAssetLibrary = CPUTAssetLibrary::GetAssetLibrary();


	m_ScreenHeight  = (float)height;
	m_ScreenWidth   = (float)width;

    // Before we can resize the swap chain, we must release any references to it.
    // We could have a "AssetLibrary::ReleaseSwapChainResources(), or similar.  But,
    // Generic "release all" works, is simpler to implement/maintain, and is not performance critical.
    pAssetLibrary->ReleaseTexturesAndBuffers();

    // Resize the CPUT-provided render target
    CPUT_DX11::ResizeWindow( width, height );

	if(GetFeatureLevel() >= D3D_FEATURE_LEVEL_11_0)
	{
		mPostProcess.OnSize(mpD3dDevice, width, height);
	}

	mpBackBuffer->RecreateRenderTarget(width, height );
	mpDepthBuffer->RecreateRenderTarget(width, height );

	g_CPUTZoomBox.OnSize(mpD3dDevice, width, height);

    // Resize any application-specific render targets here
    if( mpCamera ) mpCamera->SetAspectRatio(((float)width)/((float)height));

    pAssetLibrary->RebindTexturesAndBuffers();
}

//-----------------------------------------------------------------------------
void MySample::Render(double deltaSeconds)
{
    static int frameIndex = 0;

#ifdef AOIT_METRICS
    CPUTGPUProfilerDX11_AutoScopeProfile timerAll( mGPUTimerAll, CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() );
#endif


    CPUTRenderParametersDX renderParams( mpContext, frameIndex++ );
    renderParams.mpShadowCamera = NULL;
    renderParams.mpCamera = mpShadowCamera;
    renderParams.mpPerFrameConstants = (CPUTBuffer*)mpPerFrameConstantBuffer;
    renderParams.mpPerModelConstants = (CPUTBuffer*)mpPerModelConstantBuffer;
    int windowWidth, windowHeight;
    mpWindow->GetClientDimensions( &windowWidth, &windowHeight);
    renderParams.mWidth = windowWidth;
    renderParams.mHeight = windowHeight;

    cString SelectedItem;
	mpRenderType->GetSelectedItem(SelectedItem);

    UpdatePerFrameConstantBuffer(renderParams, deltaSeconds);
    //*******************************
    // Draw the shadow scene
    //*******************************

    // 2. Draw the shadowed scene using a standard shadow map from the light's point of view  
    // one could also use cascades shadow maps or other shadowing techniques.  We choose simple
    // shadow mapping for simplicity/demonstration purposes
    CPUTCamera *pLastCamera = mpCamera;
	if(frameIndex==1)
	{
	    mpCamera = renderParams.mpCamera = mpShadowCamera;
		mpShadowRenderTarget->SetRenderTarget( renderParams, 0, 0.0f, true );

		BeginDrawLists();
		if( mpConservatorySet ) { mpConservatorySet->RenderRecursive( renderParams, CPUT_MATERIAL_INDEX_SHADOW_CAST ); }
		if( mpindoorPlantsSet ) { mpindoorPlantsSet->RenderRecursive( renderParams, CPUT_MATERIAL_INDEX_SHADOW_CAST ); }
		if( mpoutdoorPlantsSet ) { mpoutdoorPlantsSet->RenderRecursive( renderParams, CPUT_MATERIAL_INDEX_SHADOW_CAST ); }
		if( mpBarrierSet ) { mpBarrierSet->RenderRecursive(renderParams,CPUT_MATERIAL_INDEX_SHADOW_CAST); }
		DrawSolidLists(renderParams);
		DrawTransparentLists(renderParams,true );

		mpShadowRenderTarget->RestoreRenderTarget(renderParams);

		mpInternalShadowRenderTarget->SetRenderTarget( renderParams, 0, 0.0f, true );
		BeginDrawLists();

		if( mpConservatorySet ) { mpConservatorySet->RenderRecursive(renderParams, CPUT_MATERIAL_INDEX_SHADOW_CAST); }
		if( mpindoorPlantsSet ) { mpindoorPlantsSet->RenderRecursive( renderParams, CPUT_MATERIAL_INDEX_SHADOW_CAST ); }
		if( mpoutdoorPlantsSet ) { mpoutdoorPlantsSet->RenderRecursive( renderParams, CPUT_MATERIAL_INDEX_SHADOW_CAST ); }
		DrawSolidLists(renderParams);

		mpInternalShadowRenderTarget->RestoreRenderTarget(renderParams);


	    mpCamera = renderParams.mpCamera = pLastCamera;
	}

    renderParams.mpCamera = mpCamera;
    renderParams.mpShadowCamera = mpShadowCamera;
    UpdatePerFrameConstantBuffer(renderParams, deltaSeconds);

    // Clear back buffer
    const float clearColor[] = { 0.0f, 0.5f, 1.0f, 1.0f };
    mpContext->ClearRenderTargetView( mpBackBufferRTV,  clearColor );
    mpContext->ClearDepthStencilView( mpDepthStencilView, D3D11_CLEAR_DEPTH | D3D11_CLEAR_STENCIL, 0.0f, 0);

	BeginDrawLists();

	int NodeIndex = 0;


	if(_L("Alpha Blending") == SelectedItem)
	{
		mpGlassMaterial->SetCurrentEffect(0);
		if(CPUT_CHECKBOX_CHECKED != mpSortFoliage->GetCheckboxState())
		{
			mpGrassA1Material->SetCurrentEffect(0);
			mpGrassA2Material->SetCurrentEffect(0);
			mpLeavesTreeAMaterial->SetCurrentEffect(0);
			mpPRP_ShrubSmall_DM_AMaterial->SetCurrentEffect(0);
		}
		else
		{
			mpGrassA1Material->SetCurrentEffect(1);
			mpGrassA2Material->SetCurrentEffect(1);
			mpLeavesTreeAMaterial->SetCurrentEffect(1);
			mpPRP_ShrubSmall_DM_AMaterial->SetCurrentEffect(1);
		}
		mpFenceMaterial->SetCurrentEffect(0);
	}
	else if(_L("Alpha Blending + A2C") == SelectedItem)
	{
		mpGlassMaterial->SetCurrentEffect(0);

		mpGrassA1Material->SetCurrentEffect(2);
		mpGrassA2Material->SetCurrentEffect(2);
		mpLeavesTreeAMaterial->SetCurrentEffect(2);
		mpPRP_ShrubSmall_DM_AMaterial->SetCurrentEffect(2);
		mpFenceMaterial->SetCurrentEffect(1);
	}
	else if( (_L("Pixel Synchronization OIT") == SelectedItem) && mAVSMExtensionAvailable )
	{
		int MaterialOffset = (mbPixelSync->GetCheckboxState()==CPUT_CHECKBOX_CHECKED)?3:0;
		mpNodeCount->GetSelectedItem(SelectedItem);
		if(_L("2 Nodes") == SelectedItem)
		{
			mpGlassMaterial->SetCurrentEffect(1+MaterialOffset);
			mpGrassA1Material->SetCurrentEffect(3+MaterialOffset);
			mpGrassA2Material->SetCurrentEffect(3+MaterialOffset);
			mpLeavesTreeAMaterial->SetCurrentEffect(3+MaterialOffset);
			mpPRP_ShrubSmall_DM_AMaterial->SetCurrentEffect(3+MaterialOffset);
			mpFenceMaterial->SetCurrentEffect(2+MaterialOffset);
		}
		else if(_L("4 Nodes") == SelectedItem)
		{
			mpGlassMaterial->SetCurrentEffect(2+MaterialOffset);
			mpGrassA1Material->SetCurrentEffect(4+MaterialOffset);
			mpGrassA2Material->SetCurrentEffect(4+MaterialOffset);
			mpLeavesTreeAMaterial->SetCurrentEffect(4+MaterialOffset);
			mpPRP_ShrubSmall_DM_AMaterial->SetCurrentEffect(4+MaterialOffset);
			mpFenceMaterial->SetCurrentEffect(3+MaterialOffset);
			NodeIndex=1;
		}
		else if(_L("8 Nodes") == SelectedItem)
		{
			mpGlassMaterial->SetCurrentEffect(3+MaterialOffset);
			mpGrassA1Material->SetCurrentEffect(5+MaterialOffset);
			mpGrassA2Material->SetCurrentEffect(5+MaterialOffset);
			mpLeavesTreeAMaterial->SetCurrentEffect(5+MaterialOffset);
			mpPRP_ShrubSmall_DM_AMaterial->SetCurrentEffect(5+MaterialOffset);
			mpFenceMaterial->SetCurrentEffect(4+MaterialOffset);
			NodeIndex=2;
		}

	}
	else if(_L("DX11 AOIT") == SelectedItem)
	{
		mpGlassMaterial->SetCurrentEffect(7);

		mpGrassA1Material->SetCurrentEffect(9);
		mpGrassA2Material->SetCurrentEffect(9);
		mpLeavesTreeAMaterial->SetCurrentEffect(9);
		mpPRP_ShrubSmall_DM_AMaterial->SetCurrentEffect(9);
		mpFenceMaterial->SetCurrentEffect(8);
	}

	if(_L("Alpha Blending + A2C") == SelectedItem)
	{
	    mpMSAABackBuffer->SetRenderTarget( renderParams, mpMSAADepthBuffer, 0, clearColor, true );
	}
	else
	{
		if(mpZoomBox->GetCheckboxState() == CPUT_CHECKBOX_CHECKED)
			mpBackBuffer->SetRenderTarget(  renderParams, mpDepthBuffer, 0, clearColor, true );
	}

	mpSkyBoxSprite->DrawSprite(renderParams,*mpSkyBoxMaterial);
	if( mpGroundSet ) { mpGroundSet->RenderRecursive(renderParams); }
    if( mpBarrierSet ) { mpBarrierSet->RenderRecursive(renderParams); }
    if( mpindoorPlantsSet ) { mpindoorPlantsSet->RenderRecursive(renderParams); }
	if( mpConservatorySet ) { mpConservatorySet->RenderRecursive(renderParams); }
    if( mpoutdoorPlantsSet ) { mpoutdoorPlantsSet->RenderRecursive(renderParams); }

	mpRenderType->GetSelectedItem(SelectedItem);

	// Render Solid Objects and any normal transpartent objects
	{
#ifdef AOIT_METRICS
		CPUTGPUProfilerDX11_AutoScopeProfile timer( mGPUTimerPrePass, CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() );
#endif
		DrawSolidLists(renderParams);

		if((_L("Alpha Blending") == SelectedItem) || (_L("Alpha Blending + A2C") == SelectedItem))
		{
			// Basic Pass rebder scene once with no AOIT
			DrawTransparentLists(renderParams,true );
		}
	}


	// Render AOIT Objects a
	if((_L("Pixel Synchronization OIT") == SelectedItem) || (_L("DX11 AOIT") == SelectedItem))
	{
		if(_L("Pixel Synchronization OIT") == SelectedItem)
		{
	#ifdef AOIT_METRICS
			CPUTGPUProfilerDX11_AutoScopeProfile timer( mGPUTimerAOITCreation, CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() );
	#endif
			mPostProcess.InitFrameRender(mpContext,SelectedItem, NodeIndex, (mpShowDebugView && (CPUT_CHECKBOX_CHECKED == mpShowDebugView->GetCheckboxState())));
			DrawTransparentLists(renderParams,true );
		}
		else if(_L("DX11 AOIT") == SelectedItem)
		{
	#ifdef AOIT_METRICS
			CPUTGPUProfilerDX11_AutoScopeProfile timer( mGPUTimerAOITCreation, CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() );
	#endif
			mPostProcess.InitFrameRender(mpContext,SelectedItem, 0, (mpShowDebugView && (CPUT_CHECKBOX_CHECKED == mpShowDebugView->GetCheckboxState())));
			DrawTransparentLists(renderParams,true);
		}

#ifdef AOIT_METRICS
		CPUTGPUProfilerDX11_AutoScopeProfile timer( mGPUTimerResolve, CPUT_CHECKBOX_CHECKED == mpShowStats->GetCheckboxState() );
#endif
		mPostProcess.FinalizeFrameRender(mpContext,SelectedItem,  true ,NodeIndex, (mpShowDebugView && (CPUT_CHECKBOX_CHECKED == mpShowDebugView->GetCheckboxState())));
	}

	if(_L("Alpha Blending + A2C") == SelectedItem)
	{
		mpMSAABackBuffer->RestoreRenderTarget( renderParams );

		if(mpZoomBox->GetCheckboxState() == CPUT_CHECKBOX_CHECKED)
		{
			mpBackBuffer->SetRenderTarget(  renderParams, mpDepthBuffer, 0, clearColor, true );
			mpResolveSprite->DrawSprite(renderParams, *mpResolveMaterial );
			mpBackBuffer->RestoreRenderTarget( renderParams );
		}

		mpResolveSprite->DrawSprite(renderParams, *mpResolveMaterial );
	}
	else
	{
		if(mpZoomBox->GetCheckboxState() == CPUT_CHECKBOX_CHECKED)
		{
			mpBackBuffer->RestoreRenderTarget( renderParams );
			mpFSSprite->DrawSprite(renderParams);
		}
	}	
	DisplayDebugViews(renderParams);

    if(mpCameraController->GetCamera() == mpShadowCamera)
    {
        mpDebugSprite->DrawSprite(renderParams);
    }

	if(mpZoomBox->GetCheckboxState() == CPUT_CHECKBOX_CHECKED)
		g_CPUTZoomBox.OnFrameRender( renderParams, SelectedItem, false );

	RenderText();
    CPUTDrawGUI();
}

void MySample::UpdateEnableStatsCheckbox( )
{
    bool bEnableStats = true;
    cString SelectedItem;
	mpRenderType->GetSelectedItem(SelectedItem);

    if(mpShowDebugView && ( CPUT_CHECKBOX_CHECKED == mpShowDebugView->GetCheckboxState()) )
        bEnableStats = false;

    if( mSyncInterval )// && ! CPUTGetFullscreenState() )
        bEnableStats = false;

	mbPixelSync->SetEnable(_L("Pixel Synchronization OIT") == SelectedItem);


    mpShowStats->SetEnable(bEnableStats);
}
