//--------------------------------------------------------------------------------------
// Copyright 2013 Intel Corporation
// All Rights Reserved
//
// Permission is granted to use, copy, distribute and prepare derivative works of this
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  THIS SOFTWARE IS PROVIDED "AS IS."
// INTEL SPECIFICALLY DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, AND ALL LIABILITY,
// INCLUDING CONSEQUENTIAL AND OTHER INDIRECT DAMAGES, FOR THE USE OF THIS SOFTWARE,
// INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PROPRIETARY RIGHTS, AND INCLUDING THE
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
//--------------------------------------------------------------------------------------
#ifndef __CPUT_SAMPLESTARTDX11_H__
#define __CPUT_SAMPLESTARTDX11_H__

#include <stdio.h>
#include "CPUT_DX11.h"
#include <D3D11.h>
#include <time.h>
#include "CPUTSprite.h"
#include "CPUTScene.h"
#include "CPUTBufferDX11.h"
#include "CPUTBufferDX11.h"
#include "CPUTGPUTimerDX11.h"
#include "AOITDefines.h"
#include "AOIT Technique\AOIT_Technique.h"


// define some controls
const CPUTControlID ID_MAIN_PANEL = 10;
const CPUTControlID ID_SECONDARY_PANEL = 20;
const CPUTControlID ID_FULLSCREEN_BUTTON = 100;
const CPUTControlID ID_NEXTMODEL_BUTTON = 101;
const CPUTControlID ID_VSYNC_CHECKBOX    = 102;
const CPUTControlID ID_TEST_CONTROL = 1000;
const CPUTControlID ID_IGNORE_CONTROL_ID = -1;

// AOIT Controls
const CPUTControlID ID_DEBUG_DISPLAY = 111;
const CPUTControlID ID_ENABLE_STATS = 112;
const CPUTControlID ID_RENDERTYPE = 114;
const CPUTControlID ID_DEPTH = 115;
const CPUTControlID ID_RESOLVE = 116;
const CPUTControlID ID_RENDERTRANSPARENT = 120;





const CPUTControlID ID_AOITT = 200;
const CPUTControlID ID_RT = 201;
const CPUTControlID ID_TT = 202;

struct FrameStats 
{
    float AOITCreationTime;
    float ResolveTime;
    float TotalTime;
	float PrePassTime;
}; 

//-----------------------------------------------------------------------------
class MySample : public CPUT_DX11
{

	float					m_ScreenWidth;
	float					m_ScreenHeight;
private:
    CPUTAssetSet          *mpConservatorySet;
	CPUTAssetSet          *mpGroundSet;
	CPUTAssetSet		  *mpoutdoorPlantsSet;
	CPUTAssetSet		  *mpindoorPlantsSet;
	CPUTAssetSet		  *mpBarrierSet;
    CPUTCameraController  *mpCameraController;
    CPUTSprite            *mpDebugSprite;
	CPUTSprite			  *mpSkyBoxSprite;
	CPUTSprite			  *mpFSSprite;

	CPUTMaterial			*mpGrassA1Material;
	CPUTMaterial			*mpGrassA2Material;
	CPUTMaterial			*mpLeavesTreeAMaterial;
	CPUTMaterial			*mpPRP_ShrubSmall_DM_AMaterial;
	CPUTMaterial			*mpGlassMaterial;
	CPUTMaterial			*mpFenceMaterial;

    CPUTRenderTargetColor	*mpMSAABackBuffer;
	CPUTRenderTargetDepth	*mpMSAADepthBuffer;

    CPUTText               *mpFPSCounter;
    CPUTRenderTargetDepth  *mpShadowRenderTarget;
	CPUTRenderTargetDepth		  *mpInternalShadowRenderTarget;
    
	// AOIT Stuff
    bool                mAVSMExtensionAvailable;
	AOITTechnique		mPostProcess;

    CPUTSprite            *mpDepthSprite;
	CPUTSprite            *mpResolveSprite;

    // Debug stuff
    CPUTBufferDX11 *    mpAOITDebugViewConsts;
    CPUTMaterial *      mpAOITDebugViewMaterial;
    CPUTMaterial *      mpAOITDebugDepthMaterial;
    CPUTMaterial *      mpAOITDebugMSAADepthMaterial;
	CPUTMaterial *      mpResolveMaterial;
	CPUTMaterial *		mpSkyBoxMaterial;

    CPUTCheckbox        *mpShowDebugView;
    CPUTCheckbox        *mpShowDepth;
	CPUTCheckbox		*mpZoomBox;
	CPUTCheckbox		*mbPixelSync;
	CPUTCheckbox		*mpSortFoliage;
	
#ifdef AOIT_METRICS
	CPUTCheckbox        *mpShowStats;
#endif

	CPUTDropdown		*mpNodeCount;

	CPUTGPUTimerDX11    mGPUTimerPrePass;
	CPUTGPUTimerDX11    mGPUTimerAOITCreation;
    CPUTGPUTimerDX11    mGPUTimerResolve;
    CPUTGPUTimerDX11    mGPUTimerAll;

#ifdef AOIT_METRICS
	// Stats Counters
    CPUTText			*mpAOITT;
	CPUTText			*mpRT;
	CPUTText			*mpTT;
	CPUTText			*mpPPT;
#endif

	FrameStats Stats;

    CPUTRenderTargetColor* mpBackBuffer;
    CPUTRenderTargetDepth* mpDepthBuffer;

public:
	CPUTDropdown		*mpRenderType;

    MySample() : 
		mpMSAABackBuffer(NULL),
		mpMSAADepthBuffer(NULL),
		mpShadowRenderTarget(NULL),
		mpBackBuffer(NULL),
		mpDepthBuffer(NULL),

		mpInternalShadowRenderTarget(NULL),
        mpConservatorySet(NULL),
		mpGrassA1Material(NULL),
		mpGrassA2Material(NULL),
		mpLeavesTreeAMaterial(NULL),
		mpPRP_ShrubSmall_DM_AMaterial(NULL),
		mpGlassMaterial(NULL),
		mpFenceMaterial(NULL),
		mpGroundSet(NULL),
		mpoutdoorPlantsSet(NULL),
		mpindoorPlantsSet(NULL),
		mpBarrierSet(NULL),
        mpCameraController(NULL),
		mpDebugSprite(NULL),
		mpDepthSprite(NULL),
		mpSkyBoxSprite(NULL),
		mpFSSprite(NULL),
		mpResolveSprite(NULL),
		mpSortFoliage(NULL),
        mpShowDebugView(NULL),
		mpNodeCount(NULL),
		mpRenderType(NULL),
		mpShowDepth(NULL),
        mpAOITDebugViewConsts(NULL),
		mpResolveMaterial(NULL),
		mpSkyBoxMaterial(NULL),
        mpAOITDebugViewMaterial(NULL),
        mpAOITDebugDepthMaterial(NULL),
		mpAOITDebugMSAADepthMaterial(NULL)
    {}
    virtual ~MySample()
    {
 
	}
    virtual CPUTEventHandledCode HandleKeyboardEvent(CPUTKey key, CPUTKeyState state);
    virtual CPUTEventHandledCode HandleMouseEvent(int x, int y, int wheel, CPUTMouseState state, CPUTEventID message);
    virtual void                 HandleCallbackEvent( CPUTEventID Event, CPUTControlID ControlID, CPUTControl *pControl );
    virtual void                 HandleGUIElementEvent( CPUTEventID Event, CPUTControlID ControlID, CPUTGUIElement* pElement );

    virtual void Create();
    virtual void Render(double deltaSeconds);
    virtual void Update(double deltaSeconds);
    virtual void ResizeWindow(UINT width, UINT height);
  void                 UpdateEnableStatsCheckbox( );
	void RegisterUI();
	void CreateDebugViews();
	void DisplayDebugViews(CPUTRenderParametersDX &renderParams);
	void DeleteDebugViews();
	void RenderText( );
	void CreateCameras(UINT width, UINT height);
    void LoadAssets();
	void Shutdown();
	void AddGlobalProperties();


};
#endif // __CPUT_SAMPLESTARTDX11_H__
